from rocolib.api.components import Component
from rocolib.api.composables.graph.Face import Face, Rectangle, RightTriangle
from rocolib.api.composables import GraphComposable as Graph
from rocolib.api.ports.EdgePort import EdgePort

class ServoHolder1(Component):
    _test_params = {
        'Length': 62.5
    }

    def define(self):
        self.addParameter("Length", 62.5)
        self.addInterface("SH1port", EdgePort(self, None, "Length"))
        self.addInterface("SH1star", EdgePort(self, None, "Length"))

    def assemble(self):
        l = float(self.getParameter("Length"))

        graph = Graph()
        base = Rectangle("", l - 38, 40)
        graph.addFace(base, "base")
        sideR = Rectangle("", 12, 40)
        graph.attachEdge("base.e1", sideR, "e3", prefix="sideR", angle=90)
        bottomR = Rectangle("", 19, 40)
        graph.attachEdge("sideR.e1", bottomR, "e3", prefix="bottomR", angle=-90)
        sideL = Rectangle("", 12, 40)
        graph.attachEdge("base.e3", sideL, "e1", prefix="sideL", angle=90)
        bottomL = Rectangle("", 19, 40)
        graph.attachEdge("sideL.e3", bottomL, "e1", prefix="bottomL", angle=-90)

        self.composables["graph"] = graph
        self.setInterface("SH1port", EdgePort(self, "bottomL.e3", "Length"))
        self.setInterface("SH1star", EdgePort(self, "bottomR.e1", "Length"))

if __name__ == "__main__":
    h = ServoHolder1()
    h.test()